#!/usr/bin/perl

################################################################################

# Copyright (c) 2013 by Cisco systems, Inc.
# All Rights Reserved.

################################################################################

use strict;
use Math::BigInt;

sub to_hex
{
   my ($decimal) = @_;
   my $xhigh = sprintf("%x", $decimal >> 32);
   my $xlow = sprintf("%x", $decimal & (0xffffffff));
   if ("$xhigh" eq "0")
   {
      return "$xlow";
   }
   else
   {
      return "$xhigh$xlow";
   }
}

sub to_dec
{
   my ($hex) = @_;
   my $dec = Math::BigInt->new("0x".$hex);
   return $dec;
}

################################################################################

my $date_str = `date`;
chomp($date_str);

my $cycle2ns = 0;
my $freq = 0;
my $tsc = 0;

################################################################################

my $ps_str = `ps -ef | grep runtimedebugger`;
if ($ps_str =~ /tail -n 0 -f \/arroyo\/log\/runtimedebugger.log./)
{
   print "$date_str avs_rtd is running, try later ...\n";
   exit;
}

if (! -d "/proc/calypso/tunables")
{
   print "$date_str cserver is not running, try later ...\n";
   exit;
}

my @avs_rtd_lines=split(/\n/, `avs_rtd <<EOF
dq [q__per_cpu_offset]+b478 8
dq CN::RdtscTime::s_rdtscspersec 8
dq CN::RdtscTime::s_lastUsedCpuTicks 8
quit
EOF
`);

my $look_for = "";

foreach my $line (@avs_rtd_lines)
{
   if ($line =~ /__per_cpu_offset/)
   {
      $look_for = "cycle2ns";   
      next;
   }

   if ($line =~ /s_rdtscspersec/)
   {
      $look_for = "freq";   
      next;
   }

   if ($line =~ /s_lastUsedCpuTicks/)
   {
      $look_for = "tsc";   
      next;
   }

   if ($line =~ /0xffff[0-9|a-f]+  ([0-9|a-f]+)  /)
   {
      if ($look_for eq "cycle2ns")
      {
         $cycle2ns = to_dec($1);
      }
      elsif ($look_for eq "freq")
      {
         $freq = to_dec($1);
      }
      elsif ($look_for eq "tsc")
      {
         $tsc = to_dec($1);
      }
   }
}

if ($cycle2ns eq 0 or $freq eq 0 or $tsc eq 0)
{
   print "$date_str avs_rtd failed, cycle2ns=$cycle2ns freq=$freq tsc=$tsc\n";
   exit;
}

################################################################################

my $max_val = to_dec("ffffffffffffffff");
my $overflow_val = $max_val / $cycle2ns;

################################################################################

my $mod_tsc = $tsc % $overflow_val;
my $secs_from_overflow = ($overflow_val - $mod_tsc) / $freq;
my $days_from_overflow = $secs_from_overflow / 86400;
   
print "$date_str $secs_from_overflow seconds from overflow ($days_from_overflow days)\n";

################################################################################

